# coding=utf-8
from __future__ import absolute_import
from django.test import TestCase
from mjml import settings as mjml_settings
from testprj.tools import safe_change_mjml_settings, MJMLServers, MJMLFixtures, render_tpl


class TestMJMLTCPServer(MJMLFixtures, MJMLServers, TestCase):
    SERVER_TYPE = 'tcpserver'
    _settings_manager = None

    @classmethod
    def setUpClass(cls):
        cls._settings_manager = safe_change_mjml_settings()
        cls._settings_manager.__enter__()
        mjml_settings.MJML_BACKEND_MODE = cls.SERVER_TYPE
        super(TestMJMLTCPServer, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(TestMJMLTCPServer, cls).tearDownClass()
        cls._settings_manager.__exit__(None, None, None)

    def test_simple(self):
        html = render_tpl(self.TPLS['simple'])
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        self.assertIn('20px ', html)
        self.assertIn('Test title', html)
        self.assertIn('Test button', html)

        with self.assertRaises(RuntimeError):
            render_tpl("""
                {% mjml %}
                    123
                {% endmjml %}
            """)

    def test_large_tpl(self):
        html = render_tpl(self.TPLS['with_text_context'], {
            'text': '[START]' + ('1 2 3 4 5 6 7 8 9 0 ' * 410 * 1024) + '[END]',
        })
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        self.assertIn('[START]', html)
        self.assertIn('[END]', html)

    def test_unicode(self):
        html = render_tpl(self.TPLS['with_text_context_and_unicode'], {'text': self.TEXTS['unicode']})
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        self.assertIn(u'Український текст', html)
        self.assertIn(self.TEXTS['unicode'], html)
        self.assertIn(u'©', html)



# coding=utf-8
from __future__ import absolute_import
from django.test import TestCase
from testprj.tools import render_tpl, MJMLFixtures


class TestMJMLCMDMode(MJMLFixtures, TestCase):
    def test_big_email(self):
        big_text = '[START]' + ('Big text. ' * 820 * 1024) + '[END]'
        html = render_tpl(self.TPLS['with_text_context'], {'text': big_text})
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        self.assertIn('Big text. ', html)
        self.assertIn('[START]', html)
        self.assertIn('[END]', html)
        self.assertIn('</body>', html)
        self.assertIn('</html>', html)

    def test_unicode(self):
        smile = u'\u263a'
        checkmark = u'\u2713'
        candy = u'\U0001f36d'  # b'\xf0\x9f\x8d\xad'.decode('utf-8')
        unicode_text = smile + checkmark + candy
        html = render_tpl(self.TPLS['with_text_context_and_unicode'], {'text': unicode_text})
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        self.assertIn(unicode_text, html)
        self.assertIn(u'©', html)



import os
import sys


sys.path.insert(0,  os.path.abspath(os.path.dirname((__file__))))
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'testprj.settings')
from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()



# coding=utf-8
from __future__ import absolute_import
import os
import copy
import subprocess
import time
try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse
from contextlib import contextmanager
from django.conf import settings
from django.template import Template, Context
from mjml import settings as mjml_settings
from mjml import tools


def get_mjml_version():
    env_ver = os.environ.get('MJML_VERSION', None)
    if env_ver:
        try:
            return int(env_ver.split('.')[0])
        except (ValueError, TypeError, IndexError):
            pass
    return settings.DEFAULT_MJML_VERSION


@contextmanager
def safe_change_mjml_settings():
    """
    with safe_change_mjml_settings():
        mjml_settins.MJML_EXEC_PATH = 'other value'
        ...
    # mjml settings will be restored
    ...
    """
    settings_bak = {}
    for k, v in mjml_settings.__dict__.items():
        if k[:5] == 'MJML_':
            settings_bak[k] = copy.deepcopy(v)
    tools._cache.clear()
    try:
        yield
    finally:
        for k, v in settings_bak.items():
            setattr(mjml_settings, k, v)
        tools._cache.clear()


def render_tpl(tpl, context=None):
    if get_mjml_version() >= 4:
        tpl = tpl.replace('<mj-container>', '').replace('</mj-container>', '')
    return Template('{% load mjml %}' + tpl).render(Context(context))


class MJMLServers(object):
    SERVER_TYPE = NotImplemented  # tcpserver, httpserver
    _processes = []

    @classmethod
    def _terminate_processes(cls):
        while cls._processes:
            p = cls._processes.pop()
            p.terminate()

    @classmethod
    def _start_tcp_servers(cls):
        root_dir = os.path.dirname(settings.BASE_DIR)
        tcpserver_path = os.path.join(root_dir, 'mjml', 'node', 'tcpserver.js')
        env = os.environ.copy()
        env['NODE_PATH'] = root_dir
        for host, port in mjml_settings.MJML_TCPSERVERS:
            p = subprocess.Popen([
                'node',
                tcpserver_path,
                '--port={}'.format(port),
                '--host={}'.format(host),
            ], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=env)
            cls._processes.append(p)
        time.sleep(5)

    @classmethod
    def _stop_tcp_servers(cls):
        cls._terminate_processes()

    @classmethod
    def _start_http_servers(cls):
        env = os.environ.copy()
        for server_conf in mjml_settings.MJML_HTTPSERVERS:
            parsed = urlparse(server_conf['URL'])
            host, port = parsed.netloc.split(':')
            p = subprocess.Popen([
                'mjml-http-server',
                '--host={}'.format(host),
                '--port={}'.format(port),
                '--max-body=8500kb',
            ], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=env)
            cls._processes.append(p)
        time.sleep(5)

    @classmethod
    def _stop_http_servers(cls):
        cls._terminate_processes()

    @classmethod
    def setUpClass(cls):
        super(MJMLServers, cls).setUpClass()
        if cls.SERVER_TYPE == 'tcpserver':
            cls._start_tcp_servers()
        elif cls.SERVER_TYPE == 'httpserver':
            cls._start_http_servers()
        else:
            raise RuntimeError('Invalid SERVER_TYPE: {}', cls.SERVER_TYPE)

    @classmethod
    def tearDownClass(cls):
        if cls.SERVER_TYPE == 'tcpserver':
            cls._stop_tcp_servers()
        elif cls.SERVER_TYPE == 'httpserver':
            cls._stop_http_servers()
        else:
            raise RuntimeError('Invalid SERVER_TYPE: {}', cls.SERVER_TYPE)
        super(MJMLServers, cls).tearDownClass()


class MJMLFixtures:
    TPLS = {
        'simple': """
            {% mjml %}
                <mjml>
                <mj-body>
                <mj-container>
                    <mj-section>
                        <mj-column>
                            <mj-image src="img/test.png"></mj-image>
                            <mj-text font-size="20px" align="center">Test title</mj-text>
                        </mj-column>
                    </mj-section>
                    <mj-section>
                        <mj-column>
                            <mj-button background-color="#ffcc00" font-size="15px">Test button</mj-button>
                        </mj-column>
                    </mj-section>
                </mj-container>
                </mj-body>
                </mjml>
            {% endmjml %}
        """,
        'with_text_context': """
            {% mjml %}
                <mjml>
                <mj-body>
                <mj-container>
                    <mj-section>
                        <mj-column>
                            <mj-text>{{ text }}</mj-text>
                        </mj-column>
                    </mj-section>
                </mj-container>
                </mj-body>
                </mjml>
            {% endmjml %}
        """,
        'with_text_context_and_unicode': """
            {% mjml %}
                <mjml>
                <mj-body>
                <mj-container>
                    <mj-section>
                        <mj-column>
                            <mj-text>Український текст {{ text }} ©</mj-text>
                        </mj-column>
                    </mj-section>
                </mj-container>
                </mj-body>
                </mjml>
            {% endmjml %}
        """,
    }
    SYMBOLS = {
        'smile': u'\u263a',
        'checkmark': u'\u2713',
        'candy': u'\U0001f36d',  # b'\xf0\x9f\x8d\xad'.decode('utf-8')
    }
    TEXTS = {
        'unicode': SYMBOLS['smile'] + SYMBOLS['checkmark'] + SYMBOLS['candy'],
    }



from __future__ import absolute_import
from django.conf import settings

MJML_BACKEND_MODE = getattr(settings, 'MJML_BACKEND_MODE', 'cmd')
assert MJML_BACKEND_MODE in ('cmd', 'tcpserver', 'httpserver')

# cmd backend mode configs
MJML_EXEC_CMD = getattr(settings, 'MJML_EXEC_CMD', 'mjml')
MJML_CHECK_CMD_ON_STARTUP = getattr(settings, 'MJML_CHECK_CMD_ON_STARTUP', True)

# tcpserver backend mode configs
MJML_TCPSERVERS = getattr(settings, 'MJML_TCPSERVERS', [('127.0.0.1', 28101)])
assert isinstance(MJML_TCPSERVERS, (list, tuple))
for t in MJML_TCPSERVERS:
    assert isinstance(t, (list, tuple)) and len(t) == 2 and isinstance(t[0], str) and isinstance(t[1], int)

# httpserver backend mode configs
MJML_HTTPSERVERS = getattr(settings, 'MJML_HTTPSERVERS', [{
    'URL': 'https://api.mjml.io/v1/render',
    'HTTP_AUTH': None,  # None (default) or ('login', 'password')
}])
assert isinstance(MJML_HTTPSERVERS, (list, tuple))
for t in MJML_HTTPSERVERS:
    assert isinstance(t, dict)
    assert 'URL' in t and isinstance(t['URL'], str)
    if 'HTTP_AUTH' in t:
        http_auth = t['HTTP_AUTH']
        assert isinstance(http_auth, (type(None), list, tuple))
        if http_auth is not None:
            assert len(http_auth) == 2 and isinstance(http_auth[0], str) and isinstance(http_auth[1], str)



# coding=utf-8
from __future__ import absolute_import
import unittest
import json
import requests.auth
try:
    from unittest import mock
except ImportError:
    import mock
from django.test import TestCase
from django.utils.encoding import force_bytes
from mjml import settings as mjml_settings
from mjml.tools import requests
from testprj.tools import safe_change_mjml_settings, MJMLServers, MJMLFixtures, render_tpl


class TestMJMLHTTPServer(MJMLFixtures, MJMLServers, TestCase):
    SERVER_TYPE = 'httpserver'
    _settings_manager = None

    @classmethod
    def setUpClass(cls):
        cls._settings_manager = safe_change_mjml_settings()
        cls._settings_manager.__enter__()
        mjml_settings.MJML_BACKEND_MODE = cls.SERVER_TYPE
        super(TestMJMLHTTPServer, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(TestMJMLHTTPServer, cls).tearDownClass()
        cls._settings_manager.__exit__(None, None, None)

    def test_simple(self):
        html = render_tpl(self.TPLS['simple'])
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        self.assertIn('20px ', html)
        self.assertIn('Test title', html)
        self.assertIn('Test button', html)

    def test_large_tpl(self):
        html = render_tpl(self.TPLS['with_text_context'], {
            'text': '[START]' + ('1 2 3 4 5 6 7 8 9 0 ' * 410 * 1024) + '[END]',
        })
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        self.assertIn('[START]', html)
        self.assertIn('[END]', html)

    def test_unicode(self):
        html = render_tpl(self.TPLS['with_text_context_and_unicode'], {'text': self.TEXTS['unicode']})
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        self.assertIn(u'Український текст', html)
        self.assertIn(self.TEXTS['unicode'], html)
        self.assertIn(u'©', html)

    def test_http_server_error(self):
        with self.assertRaises(RuntimeError) as cm:
            render_tpl("""
                {% mjml %}
                    <mjml>
                        <mj-body>
                            <mj-button>
                        </mj-body>
                    </mjml>
                {% endmjml %}
            """)
        self.assertIn(' Tag: mj-button Message: mj-button ', str(cm.exception))

    @mock.patch('requests.post')
    def test_http_auth(self, post_mock):
        with safe_change_mjml_settings():
            for server_conf in mjml_settings.MJML_HTTPSERVERS:
                server_conf['HTTP_AUTH'] = ('testuser', 'testpassword')

            response = requests.Response()
            response.status_code = 200
            response._content = force_bytes(json.dumps({
                'errors': [],
                'html': 'html_string',
                'mjml': 'mjml_string',
                'mjml_version': '4.5.1',
            }))
            response.encoding = 'utf-8'
            response.headers['Content-Type'] = 'text/html; charset=utf-8'
            response.headers['Content-Length'] = len(response._content)
            post_mock.return_value = response

            render_tpl(self.TPLS['simple'])

            self.assertTrue(post_mock.called)
            self.assertIn('auth', post_mock.call_args[1])
            self.assertIsInstance(post_mock.call_args[1]['auth'], requests.auth.HTTPBasicAuth)
            self.assertEqual(post_mock.call_args[1]['auth'].username, 'testuser')
            self.assertEqual(post_mock.call_args[1]['auth'].password, 'testpassword')

    @unittest.skip('to run locally')
    def test_public_api(self):
        with safe_change_mjml_settings():
            mjml_settings.MJML_HTTPSERVERS = (
                {
                    'URL': 'https://api.mjml.io/v1/render',
                    'HTTP_AUTH': ('****', '****'),
                },
            )
            html = render_tpl(self.TPLS['with_text_context_and_unicode'], {
                'text': self.TEXTS['unicode'] + ' [START]' + ('1 2 3 4 5 6 7 8 9 0 ' * 1024) + '[END]',
            })
            self.assertIn('<html ', html)
            self.assertIn('<body', html)
            self.assertIn(u'Український текст', html)
            self.assertIn(self.TEXTS['unicode'], html)
            self.assertIn(u'©', html)
            self.assertIn('[START]', html)
            self.assertIn('[END]', html)

            with self.assertRaises(RuntimeError) as cm:
                render_tpl("""
                    {% mjml %}
                        <mjml>
                            <mj-body>
                                <mj-button>
                            </mj-body>
                        </mjml>
                    {% endmjml %}
                """)
            self.assertIn(' Tag: mj-button Message: mj-button ', str(cm.exception))



# coding=utf-8
from __future__ import absolute_import
import copy
import json
try:
    from json import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError
import socket
import random
import subprocess
import tempfile
import requests
import requests.auth
from django.utils.encoding import force_str, force_bytes
from mjml import settings as mjml_settings


_cache = {}


def _mjml_render_by_cmd(mjml_code):
    if 'cmd_args' not in _cache:
        cmd_args = copy.copy(mjml_settings.MJML_EXEC_CMD)
        if not isinstance(cmd_args, list):
            cmd_args = [cmd_args]
        for ca in ('-i', '-s'):
            if ca not in cmd_args:
                cmd_args.append(ca)
        _cache['cmd_args'] = cmd_args
    else:
        cmd_args = _cache['cmd_args']

    with tempfile.SpooledTemporaryFile(max_size=(5 * 1024 * 1024)) as stdout_tmp_f:
        try:
            p = subprocess.Popen(cmd_args, stdin=subprocess.PIPE, stdout=stdout_tmp_f, stderr=subprocess.PIPE)
            stderr = p.communicate(force_bytes(mjml_code))[1]
        except (IOError, OSError) as e:
            raise RuntimeError(
                'Problem to run command "{}"\n'.format(' '.join(cmd_args)) +
                '{}\n'.format(e) +
                'Check that mjml is installed and allow permissions for execute.\n' +
                'See https://github.com/mjmlio/mjml#installation'
            )
        stdout_tmp_f.seek(0)
        stdout = stdout_tmp_f.read()

    if stderr:
        raise RuntimeError('MJML stderr is not empty: {}.'.format(force_str(stderr)))

    return force_str(stdout)


def socket_recvall(sock, n):
    data = b''
    while len(data) < n:
        packet = sock.recv(n - len(data))
        if not packet:
            return
        data += packet
    return data


def _mjml_render_by_tcpserver(mjml_code):
    if len(mjml_settings.MJML_TCPSERVERS) > 1:
        servers = list(mjml_settings.MJML_TCPSERVERS)[:]
        random.shuffle(servers)
    else:
        servers = mjml_settings.MJML_TCPSERVERS
    mjml_code_data = force_bytes(mjml_code)
    mjml_code_data = force_bytes('{:09d}'.format(len(mjml_code_data))) + mjml_code_data
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
    s.settimeout(25)
    timeouts = 0
    for host, port in servers:
        try:
            s.connect((host, port))
        except socket.timeout:
            timeouts += 1
            continue
        except socket.error:
            continue
        try:
            s.sendall(mjml_code_data)
            ok = force_str(socket_recvall(s, 1)) == '0'
            a = force_str(socket_recvall(s, 9))
            result_len = int(a)
            result = force_str(socket_recvall(s, result_len))
            if ok:
                return result
            else:
                raise RuntimeError('MJML compile error (via MJML TCP server): {}'.format(result))
        except socket.timeout:
            timeouts += 1
        finally:
            s.close()
    raise RuntimeError(
        ('MJML compile error (via MJML TCP server): no working server\n'
         'Number of servers: {total}\n'
         'Timeouts: {timeouts}').format(total=len(servers), timeouts=timeouts)
    )


def _mjml_render_by_httpserver(mjml_code):
    if len(mjml_settings.MJML_HTTPSERVERS) > 1:
        servers = list(mjml_settings.MJML_HTTPSERVERS)[:]
        random.shuffle(servers)
    else:
        servers = mjml_settings.MJML_HTTPSERVERS

    timeouts = 0
    for server_conf in servers:
        http_auth = server_conf.get('HTTP_AUTH')
        auth = requests.auth.HTTPBasicAuth(*http_auth) if http_auth else None

        try:
            response = requests.post(
                url=server_conf['URL'],
                auth=auth,
                data=force_bytes(json.dumps({'mjml': mjml_code})),
                headers={'Content-Type': 'application/json'},
                timeout=25,
            )
        except requests.exceptions.Timeout:
            timeouts += 1
            continue

        try:
            data = response.json()
        except (TypeError, JSONDecodeError):
            data = {}

        if response.status_code == 200:
            errors = data.get('errors')
            if errors:
                msgs = ['Line: {e[line]} Tag: {e[tagName]} Message: {e[message]}'.format(e=e) for e in errors]
                raise RuntimeError('MJML compile error (via MJML HTTP server): {}'.format('\n'.join(msgs)))

            return force_str(data['html'])
        else:
            msg = '[code={}, request_id={}] {}'.format(
                response.status_code,
                data.get('request_id', ''),
                data.get('message', 'Unknown error.'),
            )
            raise RuntimeError('MJML compile error (via MJML HTTP server): {}'.format(msg))

    raise RuntimeError(
        ('MJML compile error (via MJML HTTP server): no working server\n'
         'Number of servers: {total}\n'
         'Timeouts: {timeouts}').format(total=len(servers), timeouts=timeouts)
    )


def mjml_render(mjml_code):
    if mjml_settings.MJML_BACKEND_MODE == 'cmd':
        return _mjml_render_by_cmd(mjml_code)
    elif mjml_settings.MJML_BACKEND_MODE == 'tcpserver':
        return _mjml_render_by_tcpserver(mjml_code)
    elif mjml_settings.MJML_BACKEND_MODE == 'httpserver':
        return _mjml_render_by_httpserver(mjml_code)
    raise RuntimeError('Invalid settings.MJML_BACKEND_MODE "{}"'.format(mjml_settings.MJML_BACKEND_MODE))



from __future__ import absolute_import
from django.apps import AppConfig
from django.core.exceptions import ImproperlyConfigured
from mjml.tools import mjml_render
from mjml import settings as mjml_settings


def check_mjml_command():
    try:
        html = mjml_render('<mjml><mj-body><mj-container><mj-text>MJMLv3'
                           '</mj-text></mj-container></mj-body></mjml>')
    except RuntimeError:
        try:
            html = mjml_render('<mjml><mj-body><mj-section><mj-column><mj-text>MJMLv4'
                               '</mj-text></mj-column></mj-section></mj-body></mjml>')
        except RuntimeError as e:
            raise ImproperlyConfigured(e)
    if '<html ' not in html:
        raise ImproperlyConfigured(
            'mjml command returns wrong result.\n'
            'Check MJML is installed correctly. See https://github.com/mjmlio/mjml#installation'
        )


class MJMLConfig(AppConfig):
    name = 'mjml'
    verbose_name = 'Use MJML in Django templates'

    def ready(self):
        if mjml_settings.MJML_BACKEND_MODE == 'cmd' and mjml_settings.MJML_CHECK_CMD_ON_STARTUP:
            check_mjml_command()



# coding=utf-8
from __future__ import absolute_import
from django.test import TestCase
from django.template import TemplateSyntaxError
from django.core.exceptions import ImproperlyConfigured
from mjml.apps import check_mjml_command
from mjml import settings as mjml_settings
from testprj.tools import safe_change_mjml_settings, render_tpl, MJMLFixtures


class TestMJMLApps(TestCase):
    def test_check_mjml_command(self):
        with safe_change_mjml_settings():
            mjml_settings.MJML_EXEC_CMD = '/no_mjml_exec_test'
            with self.assertRaises(ImproperlyConfigured):
                check_mjml_command()

            mjml_settings.MJML_EXEC_CMD = ['python', '-c', 'print("wrong result for testing")', '-']
            with self.assertRaises(ImproperlyConfigured):
                check_mjml_command()


class TestMJMLTemplatetag(MJMLFixtures, TestCase):
    def test_simple(self):
        html = render_tpl(self.TPLS['simple'])
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        self.assertIn('20px ', html)
        self.assertIn('Test title', html)
        self.assertIn('Test button', html)

    def test_with_vars(self):
        context = {
            'title': 'Test title',
            'title_size': '20px',
            'btn_label': 'Test button',
            'btn_color': '#ffcc00'
        }
        html = render_tpl("""
            {% mjml %}
                <mjml>
                <mj-body>
                <mj-container>
                    <mj-section>
                        <mj-column>
                            <mj-image src="img/test.png"></mj-image>
                            <mj-text font-size="{{ title_size }}" align="center">{{ title }}</mj-text>
                        </mj-column>
                    </mj-section>
                    <mj-section>
                        <mj-column>
                            <mj-button background-color="{{ btn_color }}" font-size="15px">{{ btn_label }}</mj-button>
                        </mj-column>
                    </mj-section>
                </mj-container>
                </mj-body>
                </mjml>
            {% endmjml %}
        """, context)
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        for val in context.values():
            self.assertIn(val, html)

    def test_with_tags(self):
        items = ['test one', 'test two', 'test three']
        context = {
            'items': items,
        }
        html = render_tpl("""
            {% mjml %}
                <mjml>
                <mj-body>
                <mj-container>
                    <mj-section>
                        <mj-column>
                            <mj-image src="img/test.png"></mj-image>
                            <mj-text font-size="20px" align="center">Test title</mj-text>
                        </mj-column>
                    </mj-section>
                    <mj-section>
                        <mj-column>
                            {# test_comment $}
                            {% for item in items %}
                                <mj-text align="center">{{ item }}</mj-text>
                            {% endfor %}
                            <mj-button background-color="#ffcc00" font-size="15px">Test button</mj-button>
                        </mj-column>
                    </mj-section>
                </mj-container>
                </mj-body>
                </mjml>
            {% endmjml %}
        """, context)
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        for item in items:
            self.assertIn(item, html)
        self.assertNotIn('test_comment', html)

    def test_error(self):
        with self.assertRaises(TemplateSyntaxError):
            render_tpl("""
                {% mjml "var"%}
                    <mjml><mj-body><mj-container></mj-container></mj-body></mjml>
                {% endmjml %}
            """)

        with self.assertRaises(TemplateSyntaxError):
            render_tpl("""
                {% mjml var %}
                    <mjml><mj-body><mj-container></mj-container></mj-body></mjml>
                {% endmjml %}
            """, {'var': 'test'})

    def test_unicode(self):
        html = render_tpl(self.TPLS['with_text_context_and_unicode'], {'text':  self.TEXTS['unicode']})
        self.assertIn('<html ', html)
        self.assertIn('<body', html)
        self.assertIn(u'Український текст', html)
        self.assertIn(self.TEXTS['unicode'], html)
        self.assertIn(u'©', html)



import os
BASE_DIR = os.path.abspath(os.path.dirname(__file__))

SECRET_KEY = 'test'

DEBUG = True

ALLOWED_HOSTS = ['*']

INSTALLED_APPS = (
    'mjml',
    'testprj',
)

MIDDLEWARE_CLASSES = ()

ROOT_URLCONF = 'testprj.urls'

WSGI_APPLICATION = 'testprj.wsgi.application'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        # 'NAME': os.path.join(os.path.dirname(os.path.abspath(__file__)), 'db.sqlite3'),
        'NAME': ':memory:',
    }
}

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        # 'DIRS': (os.path.join(BASE_DIR, 'templates'),),
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': (
                'django.template.context_processors.request',
            ),
        },
    },
]

MJML_BACKEND = 'cmd'
MJML_EXEC_CMD = os.path.join(os.path.dirname(BASE_DIR), 'node_modules', '.bin', 'mjml')
MJML_TCPSERVERS = (
    ('127.0.0.1', 28101),
    ('127.0.0.1', 28102),
    ('127.0.0.1', 28103),
)
MJML_HTTPSERVERS = (
    {
        'URL': 'http://127.0.0.1:38101/v1/render',
    },
    {
        'URL': 'http://127.0.0.1:38102/v1/render',
    },
)

DEFAULT_MJML_VERSION = 4
